"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const utils_1 = require("./utils");
const CONSTANTS_1 = require("./CONSTANTS");
const logger = utils_1.createLogger('azure.functions.deploy.PackhostGenerator');
class PackhostGenerator {
    constructor(options) {
        this.functionsMap = new Map();
        this.options = options;
        this.options.indexFileName = this.options.indexFileName || CONSTANTS_1.DEFAULT_INDEX;
        this.options.outputPath = this.options.outputPath || CONSTANTS_1.DEFAULT_OUTPUT;
        logger.debug(`Created new PackhostGenerator for project at: ${this.options.projectRootPath}`);
    }
    updateProject() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug("Starting update of Project");
            yield this.throwIfInFunction();
            yield this.load();
            yield this.createOutputDirectory();
            yield this.createHostJSFile();
            yield this.moveMetaFiles();
            yield this.updateFunctionJSONs();
            logger.debug("Completed update of project");
        });
    }
    throwIfInFunction() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug("Checking if we're in a function");
            if (yield utils_1.FileHelper.exists(path.resolve(this.options.projectRootPath, "function.json"))) {
                throw new Error("function.json detected: run this from "
                    + "the root of your Function App, not inside of a Function");
            }
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            this.functionsMap = yield utils_1.FileHelper.loadFunctionsFromDirectory(this.options.projectRootPath);
        });
    }
    createOutputDirectory() {
        return __awaiter(this, void 0, void 0, function* () {
            const outputDirPath = path.join(this.options.projectRootPath, this.options.outputPath);
            if (yield utils_1.FileHelper.exists(outputDirPath)) {
                logger.debug(`Deleting previous output directory: ${this.options.outputPath}`);
                yield utils_1.FileHelper.rimraf(outputDirPath);
            }
            logger.debug(`Creating output directory: ${outputDirPath}`);
            yield utils_1.FileHelper.mkDirP(outputDirPath);
        });
    }
    createHostJSFile() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug("Generating host file");
            const exportStrings = [];
            const outputDirPath = path.join(this.options.projectRootPath, this.options.outputPath);
            const relPath = path.relative(outputDirPath, this.options.projectRootPath);
            const rootRelPath = (path.sep === "\\") ? relPath.replace(/\\/g, "/") : relPath;
            for (const [name, fx] of this.functionsMap) {
                const fxvar = this.safeFunctionName(fx.name);
                let exportStmt = `    "${fxvar}": require("${rootRelPath}/${name}/${fx._originalScriptFile}")`;
                logger.debug(`The export string is ${exportStmt}`);
                if (fx.entryPoint) {
                    exportStmt += `.${fx.entryPoint}`;
                }
                exportStrings.push(exportStmt);
            }
            let exportString = exportStrings.reduce((p, c, i, a) => p + c + ((i !== exportStrings.length - 1) ? ",\n" : "\n"), "");
            exportString = "module.exports = {\n" + exportString + "}";
            const hostFilePath = path.join(this.options.projectRootPath, this.options.outputPath, this.options.indexFileName);
            logger.debug(`Writing contents to host file: ${hostFilePath}`);
            yield utils_1.FileHelper.writeFileUtf8(hostFilePath, exportString);
        });
    }
    moveMetaFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug("Moving meta fils, i.e host.json, funciton.json");
            yield utils_1.FileHelper.copy(path.resolve(this.options.projectRootPath, "host.json"), path.resolve(this.options.outputPath, "host.json"));
            for (const [name, fx] of this.functionsMap) {
                logger.debug(`Moving meta for function ${name}`);
                const fxJsonPath = path.resolve(this.options.projectRootPath, name, "function.json");
                const fxJsoneTargetFolderPath = path.resolve(this.options.projectRootPath, this.options.outputPath, name);
                if (!(yield utils_1.FileHelper.exists(fxJsoneTargetFolderPath))) {
                    yield utils_1.FileHelper.mkdir(fxJsoneTargetFolderPath);
                }
                yield utils_1.FileHelper.copy(fxJsonPath, path.resolve(fxJsoneTargetFolderPath, "function.json"));
            }
        });
    }
    updateFunctionJSONs() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug("Updating function.json file");
            for (const [name, fx] of this.functionsMap) {
                logger.debug(`Updating function${name}`);
                const fxJsonPath = path.resolve(this.options.projectRootPath, this.options.outputPath, name, "function.json");
                const fxvar = this.safeFunctionName(fx.name);
                const fxJson = yield utils_1.FileHelper.readFileAsJSON(fxJsonPath);
                fxJson._originalScriptFile = fx._originalScriptFile;
                fxJson.scriptFile = `../${this.options.indexFileName}`;
                fxJson.entryPoint = fxvar;
                yield utils_1.FileHelper.overwriteFileUtf8(fxJsonPath, JSON.stringify(fxJson, null, " "));
            }
        });
    }
    safeFunctionName(name) {
        return name.replace("-", "$dash");
    }
}
exports.PackhostGenerator = PackhostGenerator;
//# sourceMappingURL=packhostGenerator.js.map