"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Promise = require("bluebird");
const ResourceManagement = require("azure-arm-resource");
const path = require("path");
const fs = require("fs");
const request = require("request");
const utils_1 = require("../utils");
const WebSiteManagementClient = require('azure-arm-website');
const dns = require('dns');
const jsonpath = require('jsonpath');
const { login } = require('az-login');
const config = require('./config');
const pkg = require('../../package.json');
const logger = utils_1.createLogger("azure.functions.deploy.azureProvider");
class AzureProvider {
    constructor(options) {
        this.existingFunctionApp = false;
        this.deployedFunctionNames = [];
        this.log = console.log;
        this.location = options.location || 'westus';
        this.functionAppName = options.functionAppName;
        this.resourceGroupName = `${this.functionAppName}-rg`;
        this.deploymentName = `${this.resourceGroupName}-deployment`;
    }
    login() {
        logger.info(`Login to azure`);
        return login({
            interactiveLoginHandler: (code, message) => {
                logger.info(message);
            }
        }).then((result) => {
            this.principalCredentials = result.credentials;
            this.subscriptionId = result.subscriptionId;
            logger.info(`Login to azure succesfully`);
            return this.principalCredentials;
        });
    }
    createResourceGroup() {
        if (!this.functionAppName) {
            throw 'Must given a unique function App name';
        }
        const groupParameters = {
            location: this.location,
            tags: { sampletag: 'sampleValue' }
        };
        logger.info(`Creating resource group: ${this.resourceGroupName}`);
        const resourceClient = new ResourceManagement.ResourceManagementClient(this.principalCredentials, this.subscriptionId);
        resourceClient.addUserAgentInfo(`${pkg.name}/${pkg.version}`);
        return new Promise((resolve, reject) => {
            resourceClient.resourceGroups.createOrUpdate(this.resourceGroupName, groupParameters, (error, result) => {
                if (error)
                    return reject(error);
                resolve(result);
            });
        }).then((result) => {
            logger.info(`Created resource group: ${this.resourceGroupName}`);
            return result;
        });
    }
    createFunctionApp() {
        logger.info(`Creating function app: ${this.functionAppName}`);
        const resourceClient = new ResourceManagement.ResourceManagementClient(this.principalCredentials, this.subscriptionId);
        let parameters = { functionAppName: { value: this.functionAppName } };
        resourceClient.addUserAgentInfo(`${pkg.name}/${pkg.version}`);
        let templateFilePath = path.join(__dirname, 'armTemplates', 'azuredeploy.json');
        let template = JSON.parse(fs.readFileSync(templateFilePath, 'utf8'));
        const deploymentParameters = {
            properties: {
                mode: 'Incremental',
                parameters,
                template
            }
        };
        return new Promise((resolve, reject) => {
            resourceClient.deployments.createOrUpdate(this.resourceGroupName, this.deploymentName, deploymentParameters, (error, result) => {
                if (error)
                    return reject(error);
                logger.info('Waiting for Kudu endpoint...');
                setTimeout(() => {
                    resolve(result);
                }, 10000);
            });
        })
            .then((result) => {
            logger.info(`Funciton App Created: ${this.resourceGroupName}`);
            return result;
        });
    }
    deleteDeployment() {
        logger.info(`Deleting deployment: ${this.deploymentName}`);
        const resourceClient = new ResourceManagement.ResourceManagementClient(this.principalCredentials, this.subscriptionId);
        resourceClient.addUserAgentInfo(`${pkg.name}/${pkg.version}`);
        return new Promise((resolve, reject) => {
            resourceClient.deployments.deleteMethod(this.resourceGroupName, this.deploymentName, (error, result) => {
                if (error)
                    return reject(error);
                resolve(result);
            });
        });
    }
    deleteResourceGroup() {
        logger.info(`Deleting resource group: ${this.resourceGroupName}`);
        const resourceClient = new ResourceManagement.ResourceManagementClient(this.principalCredentials, this.subscriptionId);
        resourceClient.addUserAgentInfo(`${pkg.name}/${pkg.version}`);
        return new Promise((resolve, reject) => {
            resourceClient.resourceGroups.deleteMethod(this.resourceGroupName, (error, result) => {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(result);
                }
            });
        });
    }
    isExistingFunctionApp() {
        const webSiteManagementClient = new WebSiteManagementClient(this.principalCredentials, this.subscriptionId, null, null);
        return webSiteManagementClient.webApps.get(this.resourceGroupName, this.functionAppName)
            .then((result) => {
            return result != null;
        })
            .then((result) => {
            logger.debug(`The function App ${this.functionAppName} existence is: ${result}`);
            this.existingFunctionApp = result;
            return result;
        })
            .catch((error) => {
            if (error.statusCode == 404) {
                return false;
            }
            else {
                throw error;
            }
        });
    }
    getDeployedFunctionsNames() {
        const requestUrl = `https://${this.functionAppName}${config.scmDomain}${config.functionsApiPath}`;
        const options = {
            host: this.functionAppName + config.scmDomain,
            method: 'get',
            url: requestUrl,
            json: true,
            headers: {
                Authorization: config.bearer + this.principalCredentials.tokenCache._entries[0].accessToken,
                Accept: 'application/json,*/*'
            }
        };
        return new Promise((resolve, reject) => {
            if (this.existingFunctionApp) {
                logger.info('Looking for deployed functions that are not part of the current deployment...');
                request(options, (err, res, body) => {
                    if (err) {
                        if (err.message.includes('ENOTFOUND')) {
                            resolve(res);
                        }
                        else {
                            reject(err);
                        }
                    }
                    else {
                        if (res.statusCode === 200) {
                            for (let functionNamesIndex = 0; functionNamesIndex < body.length; functionNamesIndex++) {
                                const name = body[functionNamesIndex].name;
                                logger.info(`Deployed function ${name} found`);
                                this.deployedFunctionNames.push(name);
                            }
                        }
                        resolve(res);
                    }
                });
            }
            else {
                resolve('New Function App ...');
            }
        });
    }
    getLogsStream(functionName) {
        const logOptions = {
            url: `https://${this.functionAppName}${config.scmDomain}${config.logStreamApiPath}${functionName}`,
            headers: {
                Authorization: config.bearer + this.principalCredentials.tokenCache._entries[0].accessToken,
                Accept: '*/*'
            }
        };
        request
            .get(logOptions)
            .on('error', () => {
            console.error('Disconnected from log streaming.');
        })
            .on('end', this.getLogsStream.bind(this))
            .pipe(process.stdout);
    }
    getInvocationId(functionName) {
        const options = {
            url: `https://${this.functionAppName}${config.scmDomain}${config.logInvocationsApiPath + this.functionAppName}-${functionName}/invocations?limit=5`,
            method: 'GET',
            json: true,
            headers: {
                Authorization: config.bearer + this.principalCredentials.tokenCache._entries[0].accessToken
            }
        };
        return new Promise((resolve, reject) => {
            request(options, (err, response, body) => {
                if (err)
                    return reject(err);
                if (response.statusCode !== 200)
                    return reject(body);
                this.invocationId = body.entries[0].id;
                resolve(body.entries[0].id);
            });
        });
    }
    getLogsForInvocationId() {
        logger.info(`Logs for InvocationId: ${this.invocationId}`);
        const options = {
            url: `https://${this.functionAppName}${config.scmDomain}${config.logOutputApiPath}${this.invocationId}`,
            method: 'GET',
            json: true,
            headers: {
                Authorization: config.bearer + this.principalCredentials.tokenCache._entries[0].accessToken
            }
        };
        return new Promise((resolve, reject) => {
            request(options, (err, response, body) => {
                if (err)
                    return reject(err);
                if (response.statusCode !== 200)
                    return reject(body);
                resolve(body);
            });
        });
    }
    cleanUpFunctionsBeforeDeploy(serverlessFunctions) {
        const deleteFunctionPromises = [];
        logger.debug(`Clean up functoins before Deploy`);
        this.deployedFunctionNames.forEach((functionName) => {
            logger.debug(`Checking function : ${functionName}`);
            if (serverlessFunctions.indexOf(functionName) < 0) {
                logger.info(`Deleting function : ${functionName}`);
                deleteFunctionPromises.push(this.deleteFunction(functionName));
            }
        });
        return Promise.all(deleteFunctionPromises);
    }
    deleteFunction(functionName) {
        const requestUrl = `https://${this.functionAppName}${config.scmVfsPath}${functionName}/?recursive=true`;
        const options = {
            host: this.functionAppName + config.scmDomain,
            method: 'delete',
            url: requestUrl,
            json: true,
            headers: {
                Authorization: config.bearer + this.principalCredentials.tokenCache._entries[0].accessToken,
                Accept: '*/*'
            }
        };
        return new Promise((resolve, reject) => {
            request(options, (err, res) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(res);
                }
            });
        });
    }
    uploadFunction(functionZipFile) {
        return new Promise((resolve, reject) => {
            logger.info(`Uploading function: ${this.functionAppName}`);
            const requestUrl = `https://${this.functionAppName}${config.scmZipApiPath}`;
            const options = {
                url: requestUrl,
                headers: {
                    Authorization: config.bearer + this.principalCredentials.tokenCache._entries[0].accessToken,
                    Accept: '*/*'
                }
            };
            logger.debug(`request url is: ${requestUrl}`);
            fs.createReadStream(functionZipFile)
                .pipe(request.post(options, (uploadZipErr, uploadZipResponse) => {
                if (uploadZipErr) {
                    logger.info(`Uploading function ${this.functionAppName} failed`);
                    reject(uploadZipErr);
                }
                else {
                    logger.info(`Uploading function ${this.functionAppName} compelete`);
                    resolve(uploadZipResponse);
                }
            }));
        });
    }
}
exports.AzureProvider = AzureProvider;
//# sourceMappingURL=azureProvider.js.map