"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const rimraf = require("rimraf");
const path = require("path");
const zip = require("archiver");
const logger_1 = require("./logger");
const CONSTANTS_1 = require("../CONSTANTS");
const logger = logger_1.createLogger('azure.functions.webpack.FileHelper');
class FileHelper {
    static readdir(path) {
        return new Promise((resolve, reject) => {
            fs.readdir(path, (err, files) => {
                if (err) {
                    return reject(err);
                }
                resolve(files);
            });
        });
    }
    static stat(path) {
        return new Promise((resolve, reject) => {
            fs.stat(path, (err, stat) => {
                if (err) {
                    return reject(err);
                }
                resolve(stat);
            });
        });
    }
    static readFileUtf8(path) {
        return new Promise((resolve, reject) => {
            fs.readFile(path, 'utf8', (err, content) => {
                if (err) {
                    return reject(err);
                }
                resolve(content);
            });
        });
    }
    static exists(path) {
        return new Promise((resolve, reject) => {
            fs.access(path, (err) => {
                resolve(!err);
            });
        });
    }
    static readFileAsJSON(path) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                const content = yield FileHelper.readFileUtf8(path);
                resolve(JSON.parse(content));
            }
            catch (err) {
                reject(err);
            }
        }));
    }
    static overwriteFileUtf8(path, content) {
        return new Promise((resolve, reject) => {
            fs.truncate(path, (err) => __awaiter(this, void 0, void 0, function* () {
                if (err) {
                    return reject(err);
                }
                yield this.writeFileUtf8(path, content).catch(reject).then(resolve, reject);
            }));
        });
    }
    static writeFileUtf8(path, content) {
        return new Promise((resolve, reject) => {
            fs.writeFile(path, content, (err) => {
                if (err) {
                    return reject(err);
                }
                resolve();
            });
        });
    }
    static mkdir(path) {
        return new Promise((resolve, reject) => {
            fs.mkdir(path, (err) => {
                if (err) {
                    return reject(err);
                }
                resolve();
            });
        });
    }
    static mkDirP(targetPath) {
        const sep = path.sep;
        const initDir = path.isAbsolute(targetPath) ? sep : '';
        const baseDir = '.';
        return Promise.resolve(targetPath.split(sep))
            .then((data) => {
            return data.reduce((promise, childDir) => {
                return promise.then((parentDir) => {
                    const curDir = path.resolve(initDir, parentDir, childDir);
                    return curDir;
                }).then((curDir) => {
                    return Promise.all([FileHelper.exists(curDir), curDir]);
                }).then(([exists, curDir]) => {
                    if (!exists) {
                        return FileHelper.mkdir(curDir)
                            .then(() => {
                            return curDir;
                        });
                    }
                    else {
                        return curDir;
                    }
                });
            }, Promise.resolve(''));
        });
    }
    static rimraf(path) {
        return new Promise((resolve, reject) => {
            rimraf(path, (err) => {
                if (err) {
                    return reject(err);
                }
                resolve();
            });
        });
    }
    static copy(source, target) {
        return new Promise(function (resolve, reject) {
            const sourceStream = fs.createReadStream(source);
            const targetStream = fs.createWriteStream(target);
            const rejectCleanup = function (err) {
                sourceStream.destroy();
                targetStream.end();
                reject(err);
            };
            sourceStream.on('error', rejectCleanup);
            targetStream.on('error', rejectCleanup);
            targetStream.on('finish', resolve);
            sourceStream.pipe(targetStream);
        });
    }
    static rename(pathOld, pathNew) {
        return new Promise((resolve, reject) => {
            fs.rename(pathOld, pathNew, (err) => {
                if (err) {
                    return reject(err);
                }
                resolve();
            });
        });
    }
    static loadFunctionsFromDirectory(projectPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const functionsMap = new Map();
            const functions = (yield FileHelper.readdir(projectPath))
                .filter((item) => __awaiter(this, void 0, void 0, function* () { return (yield FileHelper.stat(path.resolve(projectPath, item))).isDirectory(); }));
            logger.debug(`Found these directories in project root: ${functions.join(', ')}`);
            for (const item of functions) {
                if (yield FileHelper.exists(path.resolve(projectPath, item, 'function.json'))) {
                    const fn = yield FileHelper.loadFunction(path.resolve(projectPath, item));
                    if (fn !== null) {
                        functionsMap.set(item, fn);
                    }
                }
            }
            return functionsMap;
        });
    }
    static loadFunction(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let entryPoint = null;
            let scriptFile = null;
            let originalScriptFile = false;
            logger.debug(`Found function: ${name}`);
            const fxJsonPath = path.resolve(name, 'function.json');
            const fxJson = yield FileHelper.readFileAsJSON(fxJsonPath);
            if (fxJson._originalScriptFile) {
                logger.debug(`Found originalScriptFile setting: ${fxJson._originalScriptFile}`);
                scriptFile = fxJson._originalScriptFile;
                originalScriptFile = fxJson._originalScriptFile;
            }
            else if (fxJson.scriptFile && fxJson.scriptFile.endsWith('.js') && !fxJson._originalScriptFile) {
                scriptFile = fxJson.scriptFile;
                originalScriptFile = fxJson.scriptFile;
            }
            else if (fxJson.scriptFile && !fxJson.scriptFile.endsWith('.js') && !fxJson._originalScriptFile) {
                return null;
            }
            else {
                let dir = yield FileHelper.readdir(name);
                dir = dir.filter((f) => f.endsWith('.js'));
                if (dir.length === 1) {
                    scriptFile = dir[0];
                }
                else if (dir.find((v, i, o) => {
                    return v === CONSTANTS_1.DEFAULT_INDEX;
                })) {
                    scriptFile = CONSTANTS_1.DEFAULT_INDEX;
                }
                else {
                    logger.debug(`Function ${name} does not have a valid start file inside ${dir}`);
                    return null;
                }
                originalScriptFile = scriptFile;
            }
            if (fxJson.entryPoint) {
                entryPoint = fxJson.entryPoint;
            }
            logger.debug(`Loaded function(${name}) using entryPoint: ${scriptFile} - scriptFile: ${entryPoint}`);
            return Promise.resolve({
                name,
                scriptFile,
                entryPoint,
                _originalScriptFile: originalScriptFile,
            });
        });
    }
    static zipFolder(directory, outputFile) {
        return new Promise((resolve, reject) => {
            try {
                const output = fs.createWriteStream(outputFile);
                const archive = zip('zip', {
                    zlib: { level: 9 }
                });
                output.on('close', function () {
                    logger.info(archive.pointer() + ' total bytes');
                    logger.info('archiver has been finalized and the output file descriptor has closed.');
                    resolve(outputFile);
                });
                output.on('end', function () {
                    logger.info('Data has been drained');
                });
                output.on('error', function (err) {
                    throw err;
                });
                archive.on('warning', function (err) {
                    if (err.code === 'ENOENT') {
                    }
                    else {
                        throw err;
                    }
                });
                archive.on('error', function (err) {
                    throw err;
                });
                archive.pipe(output);
                archive.directory(directory, false);
                archive.finalize();
            }
            catch (error) {
                reject(error);
            }
        });
    }
}
exports.FileHelper = FileHelper;
//# sourceMappingURL=fileHelper.js.map